import 'package:mpv/cancelable/cancelable.dart';
import 'package:mpv/cancelable/cancelable_future.dart';
import 'package:mpv/view/view.dart';

abstract class Presenter<V extends View> {
  final Map<String, Cancelable> stack = {};
  V view;

  Presenter(this.view);

  addToStack(String key, Cancelable cancelable) {
    stack[key] = cancelable;
  }

  cancel(String key) {
    final cancelable = stack[key];
    if (cancelable != null) {
      stack.remove(key);
      cancelable.cancel();
    }
  }

  close() {
    view?.close();
    stack.forEach((_, cancelable) => cancelable.cancel());
  }

  Future<T> cancelable<T>(key, Future<T> future) {
    cancel(key);
    final cancelable = CancelableFuture(future);
    addToStack(key, cancelable.cancelable);
    return cancelable;
  }
}
